#!/bin/bash
#
# Test for NTG.1 compliance
#
# Copyright (C) 2021, Stephan Mueller <smueller@chronox.de>
#
# License: see LICENSE file in root directory
#
# THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, ALL OF
# WHICH ARE HEREBY DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
# OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
# USE OF THIS SOFTWARE, EVEN IF NOT ADVISED OF THE POSSIBILITY OF SUCH
# DAMAGE.
#

. $(dirname $0)/libtest.sh

create_irqs()
{
	dd if=/bin/bash of=$HOMEDIR/sp80090b.tmp oflag=sync
	rm -f $HOMEDIR/sp80090b.tmp
	sync

	local i=0
	while [ $i -lt 256 ]
	do
		echo "Test message - ignore" >&2
		i=$(($i+1))
	done

	dd if=/dev/urandom of=/dev/null bs=32 count=1
}

check_fully_seeded()
{
	if (dmesg | grep -q "lrng_es_mgr: LRNG fully seeded with")
	then
		return 0
	else
		return 1
	fi
}

check_total_seed()
{
	seed_bits=$(dmesg | grep "lrng_es_mgr: LRNG fully seeded with" | sed 's/^.* \([0-9]\+\) bits.*$/\1/')

	if [ $seed_bits -lt 440 ]
	then
		return 1
	else
		return 0
	fi
}

check_ntg1_flag()
{
	if (grep "Standards compliance" /proc/lrng_type | grep "NTG.1" |  grep -q "2022")
	then
		return 0
	else
		return 1
	fi
}

get_max_seed()
{
	local str=$1

	local outbits=0

	for i in 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30
	do
		local bits=$(dmesg | grep "$str: obtained" | tail -n $i | head -n 1 | sed 's/^.* obtained \([0-9]\+\) bits.*$/\1/')

		if [ $outbits -lt $bits ]
		then
			outbits=$bits
		fi
	done

	echo $outbits
}

check_two_es()
{
	create_irqs

	local jent_bits=$(get_max_seed "lrng_es_jent")
	local irq_bits=$(get_max_seed "lrng_es_irq")
	local cpu_bits=$(get_max_seed "lrng_es_cpu")
	local aux_bits=$(get_max_seed "lrng_es_aux")
	local sched_bits=$(get_max_seed "lrng_es_sched")

	echo_log "Jitter RNG ES seed bits: $jent_bits"
	echo_log "Interrupt ES seed bits: $irq_bits"
	echo_log "CPU ES seed bits: $cpu_bits"
	echo_log "Auxiliary ES seed bits: $aux_bits"
	echo_log "Sched ES seed bits: $sched_bits"

	found=0
	if [ $jent_bits -ge 220 ]
	then
		found=$((found+1))
	fi
	if [ $irq_bits -ge 220 ]
	then
		found=$((found+1))
	fi
	if [ $cpu_bits -ge 220 ]
	then
		found=$((found+1))
	fi
	if [ $aux_bits -ge 220 ]
	then
		found=$((found+1))
	fi
	if [ $sched_bits -ge 220 ]
	then
		found=$((found+1))
	fi

	if [ $found -ge 2 ]
	then
		return 0
	else
		return 1
	fi
}

two_es_available()
{
	$(check_ntg1_flag)
	if [ $? -eq 0 ]
	then
		echo_pass "NTG.1: standards flag present"
	else
		echo_fail "NTG.1: standards flag not present"
	fi

	$(check_fully_seeded)
	if [ $? -ne 0 ]
	then
		echo_deact "NTG.1: Fully seeding level not reached"
		return
	fi

	$(check_total_seed)
	if [ $? -eq 0 ]
	then
		echo_pass "NTG.1: Total seed equal or above 440"
	else
		echo_fail "NTG.1: Total seed below 440"
	fi

	$(check_two_es)
	if [ $? -eq 0 ]
	then
		echo_pass "NTG.1: Two ES deliver equal or more 220 bits"
	else
		echo_fail "NTG.1: Two ES do not deliver equal or more 220 bits"
	fi
}

one_es_available()
{
	$(check_ntg1_flag)
	if [ $? -eq 0 ]
	then
		echo_pass "NTG.1: standards flag present"
	else
		echo_fail "NTG.1: standards flag not present"
	fi

	$(check_fully_seeded)
	if [ $? -eq 1 ]
	then
		echo_pass "NTG.1: Fully seeding level with one ES not reached"
	else
		echo_fail "NTG.1: Fully seeding level with one ES reached"
	fi

	$(check_two_es)
	if [ $? -eq 1 ]
	then
		echo_pass "NTG.1: Two ES do not deliver equal or more 220 bits"
	else
		echo_fail "NTG.1: Two ES deliver equal or more 220 bits"
	fi
}

$(in_hypervisor)
if [ $? -eq 1 ]
then
	case $(read_cmd) in
		"test1")
			two_es_available
			;;
		"test2")
			one_es_available
			;;
		*)
			echo_fail "Test $1 not found"
			;;
	esac
else
	$(check_kernel_config "CONFIG_LRNG_RUNTIME_ES_CONFIG=y")
	if [ $? -ne 0 ]
	then
		echo_deact "NTG.1: tests skipped"
		exit
	fi

	$(check_kernel_config "CONFIG_LRNG_AIS2031_NTG1_SEEDING_STRATEGY=y")
	if [ $? -ne 0 ]
	then
		echo_deact "NTG.1: tests skipped"
		exit
	fi

	$(check_kernel_config "CONFIG_CRYPTO_FIPS=y")
	if [ $? -ne 0 ]
	then
		echo_deact "NTG.1: tests skipped"
		exit
	fi

	$(check_kernel_config "CONFIG_LRNG_RUNTIME_FORCE_SEEDING_DISABLE=y")
	if [ $? -ne 0 ]
	then
		echo_deact "NTG.1: tests skipped"
		exit
	fi

	#
	# Validating Jitter RNG and IRQ ES providing sufficient seed
	#
	# TODO this test blocks the boot process
	#
	#write_cmd "test1"
	#execvirt $(full_scriptname $0) "lrng_es_jent.jent_entropy=256 lrng_es_mgr.ntg1=1 lrng_drng_mgr.force_seeding=0"

	#
	# Validating Jitter RNG, CPU and IRQ ES providing sufficient seed
	#
	write_cmd "test1"
	execvirt $(full_scriptname $0) "lrng_es_cpu.cpu_entropy=256 lrng_es_jent.jent_entropy=256 lrng_es_mgr.ntg1=1 lrng_drng_mgr.force_seeding=0"

	#
	# Validating Jitter RNG, CPU and IRQ ES providing sufficient seed and
	# that FIPS mode does not interfere
	#
	write_cmd "test1"
	execvirt $(full_scriptname $0) "fips=1 lrng_es_cpu.cpu_entropy=256 lrng_es_jent.jent_entropy=256 lrng_es_mgr.ntg1=1 lrng_drng_mgr.force_seeding=0"

	#
	# Validating Jitter RNG, CPU and IRQ ES providing sufficient seed
	# Note: CPU ES provides full seed with a different command line option
	#
	write_cmd "test1"
	execvirt $(full_scriptname $0) "random.trust_cpu=1 lrng_es_jent.jent_entropy=256 lrng_es_mgr.ntg1=1 lrng_drng_mgr.force_seeding=0"

	#
	# Validating CPU and IRQ ES providing sufficient seed
	#
	# TODO this test blocks the boot process
	#
	#write_cmd "test1"
	#execvirt $(full_scriptname $0) "lrng_es_cpu.cpu_entropy=256 lrng_es_irq.irq_entropy=256 lrng_es_mgr.ntg1=1 lrng_drng_mgr.force_seeding=0"

	#
	# Validating CPU and IRQ ES providing sufficient seed
	# Note: CPU ES provides full seed with a different command line option
	#
	# TODO this test blocks the boot process
	#
	#write_cmd "test1"
	#execvirt $(full_scriptname $0) "random.trust_cpu=1 lrng_es_irq.irq_entropy=256  lrng_es_mgr.ntg1=1 lrng_drng_mgr.force_seeding=0"

	#
	# Validating that IRQ ES alone does not provide sufficient seed
	#
	# TODO this test blocks the boot process
	#
	#write_cmd "test2"
	#execvirt $(full_scriptname $0) "lrng_es_mgr.ntg1=1 lrng_es_cpu.cpu_entropy=0 lrng_es_jent.jent_entropy=0 lrng_es_sched.sched_entropy=256 lrng_es_irq.irq_entropy=4294967295 lrng_es_krng.krng_entropy=0"
fi
